#ifndef SCTCALIBRATIONCONTROLLER_SCTAPICALL_H
#define SCTCALIBRATIONCONTROLLER_SCTAPICALL_H

#include "SctApiException.h"
#include "SctApiAccessException.h"

#ifndef __Sct_SctApi_H_
#include "Sct_SctApi/SctApi.hh"
#endif

#include "Sct/LogicErrors.h"
#include <string>

namespace SctCalibrationController {

/**
Defines a useful macro for calling SctApi
Throws SctApiAccessException if there is a problem with the call
and InvalidArgumentError if the api is a null pointer
The call should take &st as its first argument
Incidentally, this is a poor replacement for a proper wrapper that I couldn't
be bothered to write.
*/

inline SctApiAccessException raiseSctApiException(const CORBA::Exception & exception,
                                                  const char* extraErrorText,
                                                  const char* file,
                                                  unsigned int line) {
  std::string text; 
  text += "Name["; 
  text += exception._name(); 
  text += "] Context["; 
  text += extraErrorText; 
  text += "]"; 
  return SctApiAccessException(exception, text.c_str(), file, line);
}

inline SctApiAccessException raiseLongSctApiException(const Sct_SctApi::SctApiException & exception,
                                                      const char* extraErrorText,
                                                      const char* file,
                                                      unsigned int line) {
  std::string text; 
  text += "Name["; 
  text += exception._name(); 
  text += "] Detail["; 
  text += exception.detail; 
  text += "] Context["; 
  text += extraErrorText; 
  text += "]"; 
  return SctApiAccessException(exception, text.c_str(), file, line);
}

#define APICALL( api, call, errorText ) { \
    if (CORBA::is_nil(api)) { throw InvalidArgumentError("Null pointer to SctApi", __FILE__, __LINE__); } \
    try { \
    (api)->call; \
    } catch (const Sct_SctApi::SctApiException & exe) { \
        std::cerr << "APICALL caught an Sct_SctApi::SctApiException with name [" << exe._name() << "] and detail [" << exe.detail << "] therefore [" << errorText << "]" << std::endl; \
        throw raiseLongSctApiException(exe, errorText, __FILE__, __LINE__);\
    } catch (const CORBA::Exception & exe) { \
        std::cerr << "APICALL caught unknown type of CORBA::Exception with name [" << exe._name() << "] therefore [" << errorText << "]" << std::endl; \
        throw raiseSctApiException(exe, errorText, __FILE__, __LINE__);\
    } \
}

#define APIRETCALL( api, ret, call, errorText ) { \
    if (CORBA::is_nil(api)) throw InvalidArgumentError("Null pointer to SctApi", __FILE__, __LINE__); \
    try { \
    ret=(api)->call; \
    } catch (const Sct_SctApi::SctApiException & exe) { \
        std::cerr << "APICALL caught an Sct_SctApi::SctApiException with name [" << exe._name() << "] and detail [" << exe.detail << "] therefore [" << errorText << "]" << std::endl; \
        throw raiseLongSctApiException(exe, errorText, __FILE__, __LINE__);\
    } catch (const CORBA::Exception & exe) { \
        std::cerr << "APIRETCALL caught unknown type of CORBA::Exception with name [" << exe._name() << "] therefore [" << errorText << "]" << std::endl; \
        throw raiseSctApiException(exe, errorText, __FILE__, __LINE__);\
    } \
}

}

#endif //SCTCALIBRATIONCONTROLLER_SCTAPICALL_H
