#ifndef FULLBYPASSENDCAPTEST_H
#define FULLBYPASSENDCAPTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {

  /** Full Bypass test for the endcap */

class FullBypassEndcapTest : public TestRequestImpl {
public:
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static FullBypassEndcapTest fb;
	return fb._this();
    }

    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "FullBypassEndcapTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "FullBypassEndcapTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_MASK, 3), "FullBypassEndcapTest::startTest")

        // Route mask to pipeline
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 1), "FullBypassEndcapTest::startTest")

	name = "FullBypassTest";
	testVariable = ST_ELAPSED;	// Actually vary Vdd, but no variable for that

	/*for (unsigned i=0; i<6; ++i){
	    vdd.push_back(4.0 - 0.1*i);    // 6 scan points, 4.0 -> 3.5 in 0.1 steps.
	}*/

        vdd.clear();
	vdd.push_back(4.0);

	testPoints = vdd;
	nScans = testPoints.size();
    }

    /**
      Just do a token scan for now.  Don't change Vdd as it doesn't work
    */
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
	if (CORBA::is_nil(api))
	    throw InvariantViolatedError("Api pointer is 0 in getNextScan!", __FILE__, __LINE__);

	//Sct_SctApiStatus status;
	//api->modifyABCDVar(&status, 33, vdd[index]);   // set the digital voltage.
	//if (status.returnCode != 0)
	//    throw SctApiAccessException(status.returnCode, "FullBypassEndcapTest::getNextScan", __FILE__, __LINE__);

	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr srequest = sl->tokenScan();

        Sct_SctApi::Scan_var scan = srequest->getScanCorba();
        if (CORBA::is_nil(scan)) {
          APICALL(scan, configure(ST_TOKEN, 0, 61, 1), "Failed to set Scan information") ;
        }
        return srequest;
    }
    
    virtual bool canFeedbackRealImpl() {
	return false;
    }    
    
    virtual void endTestRealImpl() {
	if (CORBA::is_nil(api)) 
	    throw InvariantViolatedError("Api pointer is 0 in endTest!", __FILE__, __LINE__);
	
/*	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, 33, 4.0);  // set the digital voltage to 4
	if (status.returnCode != 0)
	    throw SctApiAccessException(status.returnCode, "FullBypassEndcapTest::endTest", __FILE__, __LINE__);
	
	api->modifyABCDVar(&status, ST_BYPASS, 4);    // 12 chips through 2 links.
	if (status.returnCode != 0)
	    throw SctApiAccessException(status.returnCode, "FullBypassEndcapTest::endTest", __FILE__, __LINE__);
*/	
	api =  Sct_SctApi::SctApiIPC::_nil();
    }

private:
    FullBypassEndcapTest() {}
    Sct_SctApi::SctApiIPC_var api;    
    std::vector<double> vdd;
};
}
}
#endif //FULLBYPASSENDCAPTEST_H
