#ifndef SCTCALIBRATIONCONTROLLER_RXTHRESHOLDTEST_H
#define SCTCALIBRATIONCONTROLLER_RXTHRESHOLDTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "../CalibrationControllerImpl.h"
#include "Sct/LogicErrors.h"
#include "SctApi/extraScans.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {

  /** RxThresholdTest */
    
class RxThresholdTest : public TestRequestImpl {
public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static RxThresholdTest rc;
	return rc._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr the_api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(the_api);
        // No masked channels
	name = "RxThresholdTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	// put in clock / 2 mode.
	APICALL(api, modifyABCDVar(ST_FEEDTHROUGH, 0), "RxThresholdTest::startTest");
	testPoints.push_back(nTriggers);
    }
    
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
	const bool issueSoftReset = false;
	Sct_CalibrationController::ScanRequest_ptr request = sl->rawScan(nTriggers, issueSoftReset);
	Sct_SctApi::Scan_var scan = request->getScanCorba();
	
	APICALL(scan, configure(ST_RX_THRESHOLD, 0, 255, 5), "Failed to set Scan information")
        return request;
    }
    
    virtual bool canFeedbackRealImpl() {
	return true;
    }
    
    virtual void endTestRealImpl() {
      // put back in not clock/2 mode
        APICALL(api, modifyABCDVar(ST_FEEDTHROUGH, 1), "RxThresholdTest::startTest");
	api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    RxThresholdTest() {}
    Sct_SctApi::SctApiIPC_var api;    
    static const int nTriggers = 10;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_RXTHRESHOLDTEST_H
