#include "DcsData.h"
#include "Sct/LogicErrors.h"
#include <algorithm>

namespace SctData{
  DcsData::DataPoint::DataPoint(float value, OWLTime time) 
    : value(value), time(time) {
  }
  DcsData::DataPoint::DataPoint() : value (0.), time(long(0)){
  }
  DcsData::DataPoint::~DataPoint(){
  }

  std::string DcsData::getClassName() const{
    return "SctData::DcsData";
  }

  DcsData::DcsData() {}
  
  DcsData::~DcsData() {}

  void DcsData::setParameter(const std::string& name, float value, OWLTime time){
    m_data[name]=DataPoint(value, time);
  }
  
  bool DcsData::hasParameter(const std::string& name) const {
    return ( m_data.find(name) != m_data.end() );
  }

  float DcsData::getParameter(const std::string& name) const {
    return getDataPoint(name).value;
  }

  OWLTime DcsData::getTime(const std::string& name) const {
    return getDataPoint(name).time;
  }
  
  std::list <std::string> DcsData::getAllParameterNames() const{
    std::list<std::string> names;
    for (std::map<std::string, DataPoint>::const_iterator i=m_data.begin();
	 i!=m_data.end(); ++i){
      names.push_back((*i).first);
    }
    return names;
  }

  DcsData::DataPoint DcsData::getDataPoint(const std::string& name) const {
    if (!hasParameter(name)){
      std::cerr << "No such parameter " << name << std::endl;
      for ( std::map<std::string, DataPoint>::const_iterator i = m_data.begin();
	    i != m_data.end(); ++i ){
	std::cerr << "\t" << (*i).first << " = " << (*i).second.value << std::endl;
      }
      throw Sct::LogicError(string("No such parameter ")+name, __FILE__, __LINE__);
    }
    return (*m_data.find(name)).second;
  }

}
