#include <iostream>
#include <ipc/object.h>
#include <ipc/core.h>

#include "SctApi.hh"

int main(int argv, char** argc) {
  IPCCore::init(argv,argc);
  IPCPartition p("SCT");
  Sct_SctApi::SctApiIPC_var handle = 
    //p.lookup("Test_Server", "SCTSctApiServer", Sct_SctApi::SctApi::ILUClassRecord);
    p.lookup<Sct_SctApi::SctApiIPC>("Test_Server");
  if (CORBA::is_nil(handle)) {
    std::cout << "Error getting SctApi" << std::endl;
    return -1;
  }

  std::cout << "Successfully got the Api server reference?\n";

  handle->status();

  std::cout << "Get default Scan\n";

  Sct_SctApi::Scan_var scanHandle = 
    handle->createScan();

  if (CORBA::is_nil(scanHandle)) {
    std::cout << "Error getting scan\n";
  } else {
    std::cout << "Try printing scan\n";
    scanHandle->print();

  }

  std::cout << "Try getting default trigger directly\n";

  Sct_SctApi::Trigger_var triggerHandle = 
    handle->createTrigger();

  if (CORBA::is_nil(triggerHandle)) {
    std::cout << "Error getting Trigger\n";
  } else {
    std::cout << "Try printing trigger\n";
    triggerHandle->print();
//     cout << "Go to sleep a bit\n";
//     sleep(5);
//     cout << "Wake up and kill server\n";
  }

  if(!CORBA::is_nil(scanHandle)) {
    std::cout << "Try getting trigger from scan\n";
    Sct_SctApi::Trigger_var scanTrigger = scanHandle->getTrigger1();

    if (CORBA::is_nil(scanTrigger)) {
      std::cout << "Error getting Trigger\n";
    } else {
      std::cout << "Try printing trigger\n";
      scanTrigger->print();

      std::cout << "Changing trigger\n";
      scanTrigger->softCalL1A( 100, 200);

      std::cout << "Print trigger again\n";
      scanTrigger->print();

      std::cout << "Print scan\n";
      scanHandle->print();
    }

    std::cout << "Try getting another trigger from scan\n";
    scanTrigger = scanHandle->getTrigger2();

    if (CORBA::is_nil(scanTrigger)) {
      std::cout << "Error getting Trigger2\n";
    } else {
      std::cout << "Try printing trigger2\n";
      scanTrigger->print();
    }
  }

  handle->doScan(scanHandle);


  std::cout << "Destroy server\n";

  // Obsolete now? (due to _var)
  // handle->destroy();
  handle->shutdown();
  return 0;
}
