#ifdef linux
#undef linux
#endif

#include "RunController.h"
#include <rc/RCStateInfo.h>
#include <rc/RunParams.h>
#include <rc/RunInfo.h>
#include "Sct/IoExceptions.h"
#include "Sct/SctNames.h"
#include "is/isinfo.h"
#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>

namespace SctConfigurationController {
    
RunControl::RunControl(SctConfiguration::Configuration& c) : config(&c) {
}

RunControl::~RunControl(){
}
   
RunControl& RunControl::initialize(SctConfiguration::Configuration& s) {
    static RunControl rc(s);
    return rc;
}

// AJB was startAction
bool RunControl::prepareAction() {
  std::cout << "ConfigServer RC prepareAction" << std::endl;
  doSave("start");
  return true;
}

  /// was exitRunning
bool RunControl::stopEFAction() {
  std::cout << "ConfigServer RC stopAction" << std::endl;
  doSave("end");
  return true;
}

void RunControl::doSave(const std::string& suffix){
  RunParams runInfo;
  ISInfo::Status status = Sct::SctNames::getISDictionary().findValue("RunParams.RunParams", runInfo);
  if (status != ISInfo::Success) {
    Sct::IsException ise(status, "Could not get RunParams, no save of configuration!", __FILE__, __LINE__);
    ise.sendToMrs(MRS_DIAGNOSTIC);
    return;
  }

  std::ostringstream oss;
  oss << Sct::SctNames::getPersistentDir() << "/configArchive_" 
      << runInfo.run_number + "_" << suffix << ".xml.gz";
  config->saveConfiguration(oss.str());
  chmod(oss.str().c_str(), S_IRUSR | S_IRGRP);

  std::ostringstream message;
  message << "Save [" <<suffix << "]";
  Sct::SctNames::Mrs() << "SCT_CONFIG_SAVE" << MRS_INFORMATION << MRS_QUALIF("sctConf") << MRS_TEXT(message.str().c_str()) << MRS_PARAM<const char*>("Filename", oss.str().c_str()) << ENDM;
  
}

}
