package DisplayGUI;

import java.io.*;
import java.util.*;
import is.*;
import Sct.Serializable;
import Sct.IS.*;
//import SctControl.*;
import java.sql.*;
import javax.swing.JOptionPane;
import ProdDatabase.SCTDBInterface;

/*
 * SCTDBTestDownloader.java
 *
 */

/**
 *
 * @author  robinson
 */
public class SCTDBAssemblyChecker extends Thread implements SCTInfo {
    private SortedMap[] serialNumberMap = new TreeMap[sctViews];
    String uploadDir,institute,password,username,undefinedstring;
    gui guiControl;



    public SCTDBAssemblyChecker(gui parent, SortedMap[] serialNumberMap, String uploadDir, String institute, String password, String undefinedstring) {
         super("SCTDB_DOWNLOAD");
         guiControl=parent;
	 for(int i=0;i<sctViews;i++) this.serialNumberMap[i] = serialNumberMap[i];
	 this.uploadDir = uploadDir;
	 this.institute=institute;
	 this.password=password;
	 this.undefinedstring=undefinedstring;
	 username=null;

         setPriority(Thread.MIN_PRIORITY);

         }


    public boolean isOkToStart() {
    
         if(!checkVariables()) return false;

         Thread[] allThreads = new Thread[Thread.activeCount()];
         Thread.enumerate(allThreads);
         for(int i=0;i<allThreads.length;i++) {
            if(allThreads[i]!=null && allThreads[i].getName().equals("SCTDB_DOWNLOAD") && allThreads[i].isAlive()) {
                  javax.swing.JOptionPane.showMessageDialog(null,"A download from the SCT database is already in progress.\nPlease wait for this to complete before requesting another download.");
                  return false;
                  }
            }
         return true;
         }

    public void run() {

    

        List errors = new ArrayList();
	Map newModulesMap = new HashMap();
	int newModuleCount=0;

//          for(int i=0;i<sctViews;i++) {
        try {
         for(int i=0;i<4;i++) {
	    if(serialNumberMap[i].size()==0) continue; // no SctRodDaq modules in this barrel/endcap
            Map newHash = new HashMap();  // map of module sn to SCTDBAssembly info for not-yet-uploaded modules
	    Hashtable assembledMap = getAssemblyHash(SCTSerialNumbers[i]);  // map of barrel to map of module to SCTDBAssemblyInfo

	    // check if the module serial numbers are valid

	    Set unknownModuleSet = getModuleSet(serialNumberMap[i].keySet(),institute);
	    if(unknownModuleSet.size()>0) {
	          for (Iterator j = unknownModuleSet.iterator(); j.hasNext(); ) errors.add((String)j.next()+" is not located at "+institute+" and cannot be assembled.");
                  }


	    for (Iterator j=serialNumberMap[i].entrySet().iterator(); j.hasNext(); ) {  // loop through modules in this barrel
              Map.Entry e = (Map.Entry) j.next();
              String thisSerialNo = (String)e.getKey();
              ModuleCell thisCell = (ModuleCell)e.getValue();
	      if(assembledMap.containsKey(thisSerialNo)) {
	         SCTDBAssemblyInfo aInfo = (SCTDBAssemblyInfo)assembledMap.get(thisSerialNo);
                 if(thisCell.getRow()!=aInfo.getRow() || thisCell.getPosition()!=aInfo.getPos()) errors.add(thisSerialNo+" SctRodDaq/SCTDB mismatch : Row "+thisCell.getRow()+"/"+aInfo.getRow()+" Pos "+thisCell.getPosition()+"/"+aInfo.getPos());
                 }
              else {
		 newHash.put(thisSerialNo,new SCTDBAssemblyInfo(thisCell.getRow(),thisCell.getPosition()));
		 newModuleCount++;
		 }
              }
	    if(newHash.size()>0) newModulesMap.put(SCTSerialNumbers[i],newHash);
	  }
	  }catch(Exception e) {errors.add("Exception reading from database - "+e.toString());}

	  if(errors.size()>0) {
	     writeSCTDBErrors(errors);
	     return;
	     }

	  int nUploaded=0;
	  for(int i=0;i<4;i++) {
	    if(!newModulesMap.containsKey(SCTSerialNumbers[i])) continue;
	    Map mMap = (Map) newModulesMap.get(SCTSerialNumbers[i]);
	    int nmodules = mMap.size();
	    if(nmodules==0) continue;
	    Object[] options = { "Register now", "Do it later" };
            int response = JOptionPane.showOptionDialog(null, "You have "+nmodules+" new modules on "+SCTViewNames[i]+". Do you want register their positions? ", "SCT Database",JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE,null, options, options[0]);
	    if(response!=JOptionPane.YES_OPTION) return;

            if(nUploaded>0) {
		    System.out.println("SctGUI - more than one barrel/endcap has new modules? Not uploading modules for "+SCTViewNames[i]);
		    return;
		    }


            String uploadFile = writeUploadAssemblyFile(SCTSerialNumbers[i],mMap);
            if(uploadFile==null) {
	         javax.swing.JOptionPane.showMessageDialog(null,"Failed to upload assembly information","SCT Database",JOptionPane.ERROR_MESSAGE);
		 return;
		 }
            String classpath = System.getProperty("guiclasspath", null);
            if(classpath!=null) {
                String cmd = "java -cp "+classpath+" UploadAssmData "+uploadFile+" "+username+" "+password;
//		System.out.println(cmd);
		// the uploader will count for every string "Item assembled" as well as the final "DB transaction commited."
		// hence we want the #of modules plus one.
		int nCounts= nmodules+1;
                guiControl.uploader.start(nCounts, cmd, false);
                }
            nUploaded++;
	    }


    }
    public void writeSCTDBErrors(List errors) {

       File errorFile = new File(uploadDir,"SCTDBerrors.log");
       
       try {
       BufferedWriter out = new BufferedWriter(new FileWriter(errorFile));
       for (ListIterator i = errors.listIterator(); i.hasNext(); ) {
         out.write((String)i.next());
	 out.newLine();
	 }
       out.close();
       }catch(Exception e){System.err.println("SctGUI - Failed to create error log file - "+e.toString());}

       String error_msg = (errors.size()==1) ? "There is one database error\n" : "There are "+Integer.toString(errors.size())+" database errors.\n";
       StringBuffer emsg = new StringBuffer(error_msg);
       emsg.append("Please view log file at "+errorFile.getAbsolutePath()+"\n");
       emsg.append("and correct the error before uploading any further modules.");
       javax.swing.JOptionPane.showMessageDialog(null,emsg.toString(),"SCT Database",JOptionPane.ERROR_MESSAGE);
    }


    public Hashtable getAssemblyHash(String serialno) throws Exception {
        Hashtable tempHash = new Hashtable();

        Statement statement = SCTDBInterface.getInstance().connection.createStatement();
        ResultSet resultSet = statement.executeQuery("SELECT ser_no,posn from ASSM_ITEMS where assm_ser_no="+serialno);
        for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
	    String sn = resultSet.getString(1);
            tempHash.put(sn,new SCTDBAssemblyInfo(resultSet.getString(2)));
	    }
        statement.close();

	return tempHash;
	}

    public Set getModuleSet(Set moduleList, String location) throws Exception {
        Set unknownModules = new HashSet(moduleList);
//	System.out.println("no of unknown modules="+unknownModules.size());
        Statement statement = SCTDBInterface.getInstance().connection.createStatement();
//         System.out.println(sqlStat.toString());
        ResultSet resultSet = statement.executeQuery("SELECT ser_no from ITEMS where locn_name='"+location+"' AND ctype LIKE '%Module%'");
        for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
	    String sn =resultSet.getString(1);
            unknownModules.remove(sn);
	    }
        statement.close();
//	System.out.println("no of unknown modules="+unknownModules.size());
	return unknownModules;
	}


    public String writeUploadAssemblyFile(String serialno, Map moduleMap) {
        File fname = new File(uploadDir,serialno+".jassm");
        try {
	BufferedWriter out = new BufferedWriter(new FileWriter(fname));
	out.write("%Assembly");
	out.newLine();
	out.write("ASSEMBLY ITEM : "+serialno);
	out.newLine();

        String todaysDate = guiUtilities.DaveUtils.getTodaysDate();
	int nuploads = moduleMap.size();
	int count=0;
	for (Iterator i=moduleMap.entrySet().iterator(); i.hasNext(); ) {  // loop through modules in this barrel
              Map.Entry e = (Map.Entry) i.next();
              String moduleSerialNo = (String)e.getKey();
              SCTDBAssemblyInfo a = (SCTDBAssemblyInfo)e.getValue();
	      out.write("# index "+a.getIndex()+" row "+a.getRow()+" pos "+a.getPos());
	      out.newLine();
	      out.write(moduleSerialNo+"  "+Integer.toString(a.getIndex())+"  "+todaysDate);
	      count++;
	      if(count<nuploads) out.newLine();
	      }
	out.close();
	}catch(Exception e){
	   System.err.println("SctGUI - Failed to create assembly upload file - "+e.toString());
	   return null;
	   }
	return fname.getAbsolutePath();
	}



    boolean checkVariables() {


        if(guiControl.getUploaderStatus()) {
          javax.swing.JOptionPane.showMessageDialog(null,"An upload is currently in progress... cannot upload assembly information.","SCT Database Assembly Uploader",javax.swing.JOptionPane.INFORMATION_MESSAGE);
          return false;
          }
        if(institute.equals(undefinedstring)) {
          javax.swing.JOptionPane.showMessageDialog(null,"You must define your 'Upload Institute' before uploading anything... use the Preferences menu.","SCT Database Assembly Uploader",javax.swing.JOptionPane.INFORMATION_MESSAGE);
          return false;
          }
        if(password.equals(undefinedstring)) {
          javax.swing.JOptionPane.showMessageDialog(null,"You must define your SCTDB password before uploading anything... use the Preferences menu.","SCT Database Assembly Uploader",javax.swing.JOptionPane.INFORMATION_MESSAGE);
          return false;
          }
        try {
          username = ProdDatabase.GeneralUtilities.getUserName(institute);
          if(username==null) {
               javax.swing.JOptionPane.showMessageDialog(null,"No username is available for location "+institute,"SCT Database Assembly Uploader",javax.swing.JOptionPane.ERROR_MESSAGE);
               return false;
               }
        }catch(Exception db){
	 javax.swing.JOptionPane.showMessageDialog(null,"Exception "+db.toString(),"SCT Database Error",javax.swing.JOptionPane.ERROR_MESSAGE);
	 return false;
	 }

          return true;
        }





}
